import java.util.Map;

import org.armedbear.lisp.JavaObject;
import org.armedbear.lisp.LispObject;

import akka.actor.UntypedActor;
import akka.actor.ActorRef;
import akka.event.Logging;
import akka.event.LoggingAdapter;
import akka.actor.Props;
import akka.japi.Creator;
import akka.actor.ActorSystem;


public class AkkaApiSimple {

    public static void main(String[] args) {
        System.out.println("AkkaApiSimple.main >");
        ActorRef a = AkkaSingleton.createActor();
        AkkaSingleton.stop();
        System.out.println("< AkkaApiSimple.main");
    }

    public static void test() {
        System.out.println("AkkaApiSimple.test >");
        ActorRef a = AkkaSingleton.createActor();
        AkkaSingleton.stop();
        System.out.println("< AkkaApiSimple.test");
    }

}

class AkkaActorUntyped extends UntypedActor {
    protected Class<?> objectType;
    protected LispObject closure;
    protected Map<Object, Object> dependencies;

    public static Props mkProps() {
        return Props.create(AkkaActorUntyped.class);
    }

    LoggingAdapter log = Logging.getLogger(getContext().system(), this);

    public void onReceive(Object message) throws Exception {
        if (message instanceof String) {
            log.info("Received String message: {}", message);
            getSender().tell(message, getSelf());
        } else
            unhandled(message);
    }
    //
    public void preStart() {
    }
 
    public void preRestart(Throwable reason, scala.Option<Object> message) {
        for (ActorRef each : getContext().getChildren()) {
            getContext().unwatch(each);
            getContext().stop(each);
        }
        postStop();
    }
 
    public void postRestart(Throwable reason) {
        preStart();
    }
 
    public void postStop() {
    }
    //
    public void LispFactory(LispObject closure, Class<?> objectType) {
        this.objectType = objectType;
        this.closure = closure;
    }
    
    protected Object createInstance() throws Exception {
        return closure.execute(JavaObject.getInstance(this)).javaInstance();
    }
}

class AkkaSingleton {
    private static AkkaSingleton instance = null;
    static int x = 0;
    ActorSystem system = null;

    protected AkkaSingleton() {
        system = ActorSystem.create();
    }

    public static AkkaSingleton getInstance() {
        if(instance == null) {
            instance = new AkkaSingleton();
        }
        return instance;
    }

    public static void stop() {
        AkkaSingleton as = getInstance();
        as.system.shutdown();
    }

    public static ActorRef createActor() {
        AkkaSingleton as = getInstance();
        return as.system.actorOf(AkkaActorUntyped.mkProps());
    }
}
